/* ------------------------------------------------------------------------ *
 * SAS Institute, Inc.                                                      *
 *                                                                          *
 * Copyright (c) 2003, 2004 SAS Institute, Inc.  All rights reserved.       *
 *                                                                          *
 * Purpose: Contains the javascript for client side manipulation of SPM     *
 *          SVG format diagrams.                                            *
 *                                                                          *
 * Author:   Ross Allard                                                    *
 * Version:  1.0                                                            *
 * 		                    		                                        *
 * ------------------------------------------------------------------------ */


var svgSVGObj;
var svgDoc;

var SPMNSURI = "http://www.sas.com/spm";

// This is the drawing object with a tooltip (title element) associated with it.
var toolTipObj;

// This is the actual tooltip part of the drawing. It's identified with the 
// id: SPM_TYPE_TOOLTIP_ID
var toolTip; 
    
// Coordinates used to determine if the mouse has been stationary. 
var toolTipXOld;
var toolTipYOld;
var toolTipXNew;
var toolTipYNew;

// Flag to keep track of the tooltip.
var toolTipDisplayed = false;

// This is used to create a short delay before displaying tooltips.  It's the time stamp of the 
// last mouse move.
var	lastMoveTime;  

// This is the tooltip delay constant, in millisecs.
var delay = 1000;

var scale;
var translate; 


function onLoad(evt) {

	if (evt == null)
		return;
		
	// Retrieve the SVG document object:
	svgSVGObj = evt.getTarget();
	svgDoc = svgSVGObj.getOwnerDocument();

	
	// SPM_TYPE_TOOLTIP_ID == "spm_tooltip".  See GfxComponent.
	toolTip = svgDoc.getElementById("spm_tooltip");
	
	
	var root = svgDoc.getDocumentElement();

		
	// Find the root <g> element and reset the transform attribute
 	var node = root.getFirstChild();
 	var gotit = "false";
 	while (node != null)	{
 		if (node.getNodeName() == "g") { 	 		
 			gotit = "true";
 			break;
 		}
 	
 		node  = node.getNextSibling();
 	}
 	
 	if (gotit == "true") 
 		firstG = node;
	
	restoreHighlight();
}


function onZoom (evt)
{
	
	if (evt == null)
		return;

	var scale = svgSVGObj.getCurrentScale();     
    var translate = svgSVGObj.getCurrentTranslate(); 
 	 	
} 

function onMouseOver ( evt )
{
	
	if (evt == null)
		return;

	// We've just entered a diagram object.  Save the object reference, 
	// current mouse position, and current time.  
	toolTipObj = evt.getTarget();

	// Find parent node. 

	var p = toolTipObj.getParentNode();
	toolTipObj = null;
	while (p != null) {
	
	  // Check the type of node to avoid type exceptions.
	  if (p.getNodeType() == 1) {
	  
	      var type = p.getAttribute("SASSPM:spmType" );
	      if (type != null && type == "node"  || type == "link") {
			toolTipObj = p;		
			break;
	      }
      }
	  p =p.getParentNode();
	}
	
	if (toolTipObj == null) {
		return;
	}
	toolTipX = evt.getClientX();
	toolTipY = evt.getClientY();
	var d = new Date();
	lastMoveTime = d.getTime();
	setTimeout('checkTooltip()', delay);
	return;

} 



function onMouseOverThreshold ( evt )
{
	
	if (evt == null)
		return;

    evt.stopPropagation();

	// We've just entered a threshold flag image.  Save the object reference, 
	// current mouse position, and current time.  
	toolTipObj = evt.getTarget();

	toolTipX = evt.getClientX();
	toolTipY = evt.getClientY();
	var d = new Date();
	lastMoveTime = d.getTime();
	setTimeout('checkTooltip()', delay);
	return;

} 

function doDirective ( evt )
{

	if (evt == null)
		return;

    evt.stopPropagation();

	var o = evt.getTarget();
	

	var list = o.getElementsByTagName("target");
	var t = list.item(0);

	list = o.getElementsByTagName("windowtarget");
	var target = list.item(0);

	var url = t.getFirstChild().getNodeValue();
	var targ = target.getFirstChild().getNodeValue();
		
	doDirective2(url, targ);

} 


function onMouseOverShape ( evt )
{

} 


function checkTooltip()
{
	if (toolTipObj == null || toolTipDisplayed == true)
		return;
		
    if (toolTipObj.getNodeType() != 1)
    	return;		
		
	// Get the  current time.  If it's one second past the last mousemove 
	// time, put up the tooltip.

	
	var d = new Date();
	var curtime = d.getTime();
	if 	( curtime - lastMoveTime > delay) {

		// Set the tooltip text to the text of the current object.
		var list = toolTipObj.getElementsByTagName("title");
		
		// There's only one. 
		var toolTipTextElement = list.item(0);
		if (toolTipTextElement == null || toolTipTextElement.getFirstChild() == null) {
			return;
		}
		var toolTipText = toolTipTextElement.getFirstChild().getNodeValue();

		list = toolTip.getElementsByTagName("text");
		
		// There's only one. 
		var toolTipElement = list.item(0);
		
		toolTipElement.getFirstChild().setNodeValue(toolTipText);
		

		// Get the current scale and translate.
		var scale = svgSVGObj.getCurrentScale(); 
		var inverse = 1 / scale;    
	    var translate = svgSVGObj.getCurrentTranslate(); 
	    
	    var xTrans = translate.getX();
   	    var yTrans = translate.getY();
   	    
   	    var newX = (toolTipX - xTrans)  * inverse + 10;
   	    var newY = (toolTipY - yTrans)  * inverse - 10;
   	    
   	    var newScale = 1 / scale;
		
		// Move it to the mouse position and make visible.
		toolTip.setAttribute("x", newX + 10);
		toolTip.setAttribute("y", newY - 10);
		toolTip.setAttribute("transform", "translate(" +  newX + "," + newY + ") " + "scale(" + newScale + ")");
		
		// Set the rectangle's width so that it encloses the current string.
		// Get the rectangle. 
		list = toolTip.getElementsByTagName("rect");
		
		// There's only one. 
		var rectElement = list.item(0);
		rectElement.setAttribute("width", toolTipElement.getComputedTextLength() + 12);
		
	    toolTip.setAttribute("visibility", "visible");
		
		// Now that we've displayed it, make sure we don't try to display it
		// again.
		lastMoveTime = curtime;
		
	    toolTipDisplayed = true;
 		return;
	}	
}

function onMouseMoveShape ( evt )
{

	if (evt == null)
		return;
}

function onMouseMove ( evt )
{

	
	if (evt == null)
		return;

	// If we're not tracking a tool tip, ignore.
	if (toolTipObj == null || toolTipDisplayed == true)
		return;

	// Reset the current time.
	var d = new Date();
	toolTipTime = d.getTime();
	
	// Save the mouse position.
	toolTipX = evt.getClientX;
	toolTipY = evt.getClientY;
	
	setTimeout('checkTooltip()', delay);
	return;

} 

function onMouseOutShape ( evt )
{
}

function onMouseOut ( evt )
{
	// Hide the tooltip.  
	toolTipObj = null;
	toolTipDisplayed = false;
	if (toolTip == null)
		return;
	toolTip.setAttribute("visibility", "hidden");
	
	return;

} 

function onClickShape ( evt )
{
	if (evt == null)
		return;
		
	var o = evt.target;
	var p = o.getParentNode();
	var node;
	try {
		while (p != null) {
	
		  // Check the type of node to avoid type exceptions.
		  if (p.getNodeType() == 1) {
	
		      var type = p.getAttribute("SASSPM:spmType" );
		      if (type != null && type == "shape") {
				node = p;		
				break;
		      }
		      if (type != null && type == "node") {
		      	break;
		      }
		  }
		  p =p.getParentNode();
		}
	} catch (ex) {
		return;
	}
	if (node == null)
		return;
	
	// Get the href and target attributes.	
	var href = node.getAttribute("SASSPM:spmHREF");
	var target = node.getAttribute("SASSPM:spmTarget");

	if (href == null || href == "")
		return;
		
    if (target == null || target == "")
    	target = "_new";

	myNavigateSVG(href, target);

}

var elementDetails="/ElementAttribute.do?Spm_elementdetails_id=";

function onClick ( evt )
{
	
	if (evt == null)
		return;
		
	// The user has clicked on an object. 
	var o = evt.getTarget();

	// Traverse the parents until we get to an SPM element node. 

	var p = o.getParentNode();
	var node;
	while (p != null) {
		  // Check the type of node to avoid type exceptions.
		  if (p.getNodeType() == 1) {
		
		      var type = p.getAttribute("SASSPM:spmType" );
		      if (type != null && type == "node") {
				node = p;		
				break;
		      }
	      }
	     p =p.getParentNode();
	}
	if (node == null)
		return;
		
	// Get the element id.
	var id = node.getAttribute("SASSPM:spmElementID");
	
	if (id == null || id == "") 
		return;
		
	
	elementDetailsSvg(id, "_self");
	
	return;
} 


var gthresholdUrl = "/ThresholdConditions.do?type=global&elementId=";
var pthresholdUrl = "/ThresholdConditions.do?type=personal&elementId=";

function doGlobalThreshold ( evt )
{
	
	if (evt == null)
		return;

    evt.stopPropagation();

	// The user has clicked on a threshold flag.. 
	var o = evt.getTarget();

	// Traverse the parents until we get to an SPM element node. 

	var p = o.getParentNode();
	var node;
	while (p != null) {
		  // Check the type of node to avoid type exceptions.
		  if (p.getNodeType() == 1) {
		
		      var type = p.getAttribute("SASSPM:spmType" );
		      if (type != null && type == "node") {
				node = p;		
				break;
		      }
	      }
	     p =p.getParentNode();
	}
	if (node == null)
		return;
		
	// Get the element id.
	var id = node.getAttribute("SASSPM:spmElementID");
	
	if (id == null || id == "") 
		return;

	var cid = o.getAttribute("SASSPM:spmColumnId");
	var pid = o.getAttribute("SASSPM:spmPeriodId");
		
	// Redirect the browser to the threshold page.
	var url = gthresholdUrl;
	url = url + id;
	url = url + "&columnId=" + cid;
	url = url + "&periodId=" + pid;

	doThreshold("global", id, cid, pid);

	return;
} 

function doPersonalThreshold ( evt )
{
	
	if (evt == null)
		return;

    evt.stopPropagation();

	// The user has clicked on a threshold flag.. 
	var o = evt.getTarget();

	// Traverse the parents until we get to an SPM element node. 

	var p = o.getParentNode();
	var node = null;
	while (p != null) {
	
		  // Check the type of node to avoid type exceptions.
		  if (p.getNodeType() == 1) {
		
		      var type = p.getAttribute("SASSPM:spmType" );
		      if (type != null && type == "node") {
				node = p;		
				break;
		      }
	      }
	     p =p.getParentNode();
	}
	if (node == null)
		return;
		
	// Get the element id.
	var id = node.getAttribute("SASSPM:spmElementID");
	
	if (id == null || id == "") 
		return;

	var cid = o.getAttribute("SASSPM:spmColumnId");
	var pid = o.getAttribute("SASSPM:spmPeriodId");
		
	// Redirect the browser to the threshold page.
	var url = pthresholdUrl;
	url = url + id;
	url = url + "&columnId=" + cid;
	url = url + "&periodId=" + pid;

	
    doThreshold("personal", id, cid, pid);
	return;
} 

function doComments ( evt )
{
	
	if (evt == null)
		return;

    evt.stopPropagation();

	// The user has clicked on a comments icon.. 
	var o = evt.getTarget();

	// Traverse the parents until we get to an SPM element node. 

	var p = o.getParentNode();
	var node = null;
	while (p != null) {
		  // Check the type of node to avoid type exceptions.
		  if (p.getNodeType() == 1) {
		
		      var type = p.getAttribute("SASSPM:spmType" );
		      if (type != null && type == "node") {
				node = p;		
				break;
		      }
	      }
	     p =p.getParentNode();
	}
	if (node == null)
		return;
		
	// Get the element id.
	var id = node.getAttribute("SASSPM:spmElementID");
	
	if (id == null || id == "") 
		return;

	var sid = node.getAttribute("SASSPM:spmScorecardID");

	// Get the element text.  It's the <title> child element.
	var list = node.getElementsByTagName("title");
	
	// There's only one. 
	var toolTipTextElement = list.item(0);
	if (toolTipTextElement == null || toolTipTextElement.getFirstChild() == null) {
		return;
	}

	doComments2( id, sid);
    
	return;
} 

function doTrend ( evt )
{
	
	if (evt == null)
		return;

    evt.stopPropagation();

	// The user has clicked on a comments icon.. 
	var o = evt.getTarget();

	// Traverse the parents until we get to an SPM element node. 

	var p = o.getParentNode();
	var node = null;
	while (p != null) {
		  // Check the type of node to avoid type exceptions.
		  if (p.getNodeType() == 1) {
	
		      var type = p.getAttribute("SASSPM:spmType" );
		      if (type != null && type == "node") {
				node = p;		
				break;
		      }
	     }
	     p =p.getParentNode();
	}
	if (node == null)
		return;
		
	// Get the element id.
	var id = node.getAttribute("SASSPM:spmElementID");
	
	if (id == null || id == "") 
		return;
	
	var sid = node.getAttribute("SASSPM:spmScorecardID");


	var pid = o.getAttributeNS(SPMNSURI,"spmPeriodId");

	// Get the element text.  It's the <title> child element.
	var list = node.getElementsByTagName("title");
	
	// There's only one. 
	var toolTipTextElement = list.item(0);
	if (toolTipTextElement == null || toolTipTextElement.getFirstChild() == null) {
		return;
	}
	var name = toolTipTextElement.getFirstChild().getNodeValue();

	doTrend2(name, id, pid, sid);
    
	return;
} 
